package xyz.matj1.bp.themes;

import org.jfree.chart.ChartTheme;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.ui.RectangleInsets;
import xyz.matj1.bp.Config;
import xyz.matj1.bp.Config.Theme;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontFormatException;
import java.io.IOException;

public class XyGraphTheme implements ChartTheme {
	private static final XyGraphTheme instance = new XyGraphTheme();

	protected XyGraphTheme() {}

	@Override
	public void apply(JFreeChart chart) {
		try {
			this.applyFonts(Config.graphFont(), chart);
		} catch (IOException | FontFormatException e) {
			System.err.println("The font for graphs couldn't be loaded because of something having thrown this exception:");
			e.printStackTrace();
		}

		chart.setPadding(RectangleInsets.ZERO_INSETS);
		chart.getPlot().setBackgroundPaint(Color.WHITE);
		chart.setBorderVisible(false);

		if (Config.graphTheme().equals(Theme.DARK)) {
			StandardChartTheme.createDarknessTheme().apply(chart);
	}}

	/**
	 * This sets the theme to the default theme but with
	 * fonts derived from <code>font</code> as its fonts.
	 * This is intended to apply fonts to the  theme, but it changes also other things,
	 * so I recommend using this before applying other changes to the theme.
	 * @param font
	 * @param chart
	 */
	public void applyFonts(Font font, JFreeChart chart) {
		var chartTheme = (StandardChartTheme)StandardChartTheme.createJFreeTheme();

		var oldExtraLargeFont = chartTheme.getExtraLargeFont();
		var oldLargeFont = chartTheme.getLargeFont();
		var oldRegularFont = chartTheme.getRegularFont();
		var oldSmallFont = chartTheme.getSmallFont();

		var extraLargeFont = font.deriveFont(oldExtraLargeFont.getStyle(), oldExtraLargeFont.getSize());
		var largeFont = font.deriveFont(oldLargeFont.getStyle(), oldLargeFont.getSize());
		var regularFont = font.deriveFont(oldRegularFont.getStyle(), oldRegularFont.getSize());
		var smallFont = font.deriveFont(oldSmallFont.getStyle(), oldSmallFont.getSize());

		chartTheme.setExtraLargeFont(extraLargeFont);
		chartTheme.setLargeFont(largeFont);
		chartTheme.setRegularFont(regularFont);
		chartTheme.setSmallFont(smallFont);

		chartTheme.apply(chart);
	}

	public static XyGraphTheme instance() {
		return instance;
	}
}
